(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
    typeof define === 'function' && define.amd ? define('@s-libs/micro-dash', ['exports'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory((global.sLibs = global.sLibs || {}, global.sLibs.microDash = {})));
}(this, (function (exports) { 'use strict';

    /**
     * Creates an array of elements split into groups the length of `size`. If `array` can't be split evenly, the final chunk will be the remaining elements.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 2,206 bytes
     * - Micro-dash: 177 bytes
     */
    function chunk(array, size) {
        if (size === void 0) { size = 1; }
        size = Math.max(Math.trunc(size), 0);
        var chunks = [];
        for (var i = 0; i < array.length; i += Math.max(1, size)) {
            chunks.push(array.slice(i, i + size));
        }
        return chunks;
    }

    /**
     * Creates a function that returns `value`.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 48 bytes
     * - Micro-dash: 39 bytes
     */
    function constant(value) {
        return function () { return value; };
    }

    /**
     * This method returns the first argument it receives.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 39 bytes
     * - Micro-dash: 39 bytes
     */
    function identity(value) {
        return value;
    }

    function flow() {
        var funcs = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            funcs[_i] = arguments[_i];
        }
        if (funcs.length) {
            return funcs.reduce(function (result, func) { return function (input) { return func(result(input)); }; });
        }
        else {
            return identity;
        }
    }

    function flowRight() {
        var funcs = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            funcs[_i] = arguments[_i];
        }
        if (funcs.length) {
            return funcs.reduce(function (result, func) { return function (input) { return result(func(input)); }; });
        }
        else {
            return identity;
        }
    }

    /**
     * Casts `value` as an array if it's not one.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 128 bytes
     * - Micro-dash: 71 bytes
     */
    function castArray(value) {
        return (Array.isArray(value) ? value : [value]);
    }

    /**
     * Creates a shallow clone of `value`.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 12,221 bytes
     * - Micro-dash: 111 bytes
     */
    function clone(value) {
        if (Array.isArray(value)) {
            return value.slice();
        }
        else if (value instanceof Object) {
            return Object.assign({}, value);
        }
        else {
            return value;
        }
    }

    function forEach(collection, iteratee) {
        if (Array.isArray(collection)) {
            forEachOfArray(collection, iteratee);
        }
        else {
            forOwnOfNonArray(collection, iteratee);
        }
        return collection;
    }
    /** @hidden */
    function forEachOfArray(array, iteratee) {
        for (var i = 0, len = array.length; i < len; ++i) {
            if (iteratee(array[i], i) === false) {
                break;
            }
        }
    }

    /**
     * Creates an array of the own enumerable property names of object.
     *
     * Differences from lodash:
     * - does not give any special consideration for arguments objects, strings, or prototype objects (e.g. many will have `'length'` in the returned array)
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 3,325 bytes
     * - Micro-dash: 164 bytes
     */
    function keys(object) {
        var val = keysOfNonArray(object);
        if (Array.isArray(object)) {
            val = val.filter(function (item) { return item !== 'length'; });
        }
        return val;
    }
    /** @hidden */
    function keysOfNonArray(object) {
        return object ? Object.getOwnPropertyNames(object) : [];
    }

    /**
     * Iterates over own enumerable string keyed properties of an object and invokes `iteratee` for each property. Iteratee functions may exit iteration early by explicitly returning `false`.
     *
     * Differences from lodash:
     * - does not treat sparse arrays as dense
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 3,513 bytes
     * - Micro-dash: 212 bytes
     */
    function forOwn(object, iteratee) {
        forEachOfArray(keys(object), function (key) { return iteratee(object[key], key); });
        return object;
    }
    /** @hidden */
    function forOwnOfNonArray(object, iteratee) {
        forEachOfArray(keysOfNonArray(object), function (key) { return iteratee(object[key], key); });
        return object;
    }

    function findKey(object, predicate) {
        var found;
        forOwn(object, function (value, key) {
            if (predicate(value, key)) {
                found = key;
                return false;
            }
        });
        return found;
    }

    function forEachRight(collection, iteratee) {
        if (Array.isArray(collection)) {
            forEachRightOfArray(collection, iteratee);
        }
        else {
            forOwnRightOfNonArray(collection, iteratee);
        }
        return collection;
    }
    /** @hidden */
    function forEachRightOfArray(array, iteratee) {
        for (var i = array.length; --i >= 0;) {
            if (iteratee(array[i], i) === false) {
                break;
            }
        }
    }

    /**
     * This method is like `forOwn` except that it iterates over properties of `object` in the opposite order.
     *
     * Differences from lodash:
     * - does not treat sparse arrays as dense
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 3,506 bytes
     * - Micro-dash: 188 bytes
     */
    function forOwnRight(object, iteratee) {
        forEachRightOfArray(keys(object), function (key) { return iteratee(object[key], key); });
        return object;
    }
    /** @hidden */
    function forOwnRightOfNonArray(object, iteratee) {
        forEachRightOfArray(keysOfNonArray(object), function (key) { return iteratee(object[key], key); });
        return object;
    }

    /**
     * Creates an array of function property names from own enumerable properties of `object`.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 3,503 bytes
     * - Micro-dash: 174 bytes
     */
    function functions(obj) {
        return Object.getOwnPropertyNames(obj).filter(function (key) { return key !== 'constructor' &&
            isFunction(Object.getOwnPropertyDescriptor(obj, key).value); });
    }

    function get(object, path, defaultValue) {
        // const val = property(path)(object);
        // return isUndefined(val) ? defaultValue : val;
        path = castArray(path);
        var length = path.length;
        var index = 0;
        while (object != null && index < length) {
            object = object[path[index++]];
        }
        return !index || index < length || object === undefined
            ? defaultValue
            : object;
    }

    function invoke(object, path) {
        var args = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            args[_i - 2] = arguments[_i];
        }
        var fn = get(object, path);
        if (isFunction(fn)) {
            return fn.apply(path.length === 1 ? object : get(object, path.slice(0, -1)), args);
        }
    }

    function mapValues(object, iteratee) {
        var obj = {};
        forOwn(object, function (value, key) {
            obj[key] = iteratee(value, key);
        });
        return obj;
    }

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation.

    Permission to use, copy, modify, and/or distribute this software for any
    purpose with or without fee is hereby granted.

    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
    PERFORMANCE OF THIS SOFTWARE.
    ***************************************************************************** */
    /* global Reflect, Promise */
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b)
                if (Object.prototype.hasOwnProperty.call(b, p))
                    d[p] = b[p]; };
        return extendStatics(d, b);
    };
    function __extends(d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    }
    var __assign = function () {
        __assign = Object.assign || function __assign(t) {
            for (var s, i = 1, n = arguments.length; i < n; i++) {
                s = arguments[i];
                for (var p in s)
                    if (Object.prototype.hasOwnProperty.call(s, p))
                        t[p] = s[p];
            }
            return t;
        };
        return __assign.apply(this, arguments);
    };
    function __rest(s, e) {
        var t = {};
        for (var p in s)
            if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
                t[p] = s[p];
        if (s != null && typeof Object.getOwnPropertySymbols === "function")
            for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
                if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                    t[p[i]] = s[p[i]];
            }
        return t;
    }
    function __decorate(decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function")
            r = Reflect.decorate(decorators, target, key, desc);
        else
            for (var i = decorators.length - 1; i >= 0; i--)
                if (d = decorators[i])
                    r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    }
    function __param(paramIndex, decorator) {
        return function (target, key) { decorator(target, key, paramIndex); };
    }
    function __metadata(metadataKey, metadataValue) {
        if (typeof Reflect === "object" && typeof Reflect.metadata === "function")
            return Reflect.metadata(metadataKey, metadataValue);
    }
    function __awaiter(thisArg, _arguments, P, generator) {
        function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
        return new (P || (P = Promise))(function (resolve, reject) {
            function fulfilled(value) { try {
                step(generator.next(value));
            }
            catch (e) {
                reject(e);
            } }
            function rejected(value) { try {
                step(generator["throw"](value));
            }
            catch (e) {
                reject(e);
            } }
            function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
            step((generator = generator.apply(thisArg, _arguments || [])).next());
        });
    }
    function __generator(thisArg, body) {
        var _ = { label: 0, sent: function () { if (t[0] & 1)
                throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
        return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function () { return this; }), g;
        function verb(n) { return function (v) { return step([n, v]); }; }
        function step(op) {
            if (f)
                throw new TypeError("Generator is already executing.");
            while (_)
                try {
                    if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done)
                        return t;
                    if (y = 0, t)
                        op = [op[0] & 2, t.value];
                    switch (op[0]) {
                        case 0:
                        case 1:
                            t = op;
                            break;
                        case 4:
                            _.label++;
                            return { value: op[1], done: false };
                        case 5:
                            _.label++;
                            y = op[1];
                            op = [0];
                            continue;
                        case 7:
                            op = _.ops.pop();
                            _.trys.pop();
                            continue;
                        default:
                            if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) {
                                _ = 0;
                                continue;
                            }
                            if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) {
                                _.label = op[1];
                                break;
                            }
                            if (op[0] === 6 && _.label < t[1]) {
                                _.label = t[1];
                                t = op;
                                break;
                            }
                            if (t && _.label < t[2]) {
                                _.label = t[2];
                                _.ops.push(op);
                                break;
                            }
                            if (t[2])
                                _.ops.pop();
                            _.trys.pop();
                            continue;
                    }
                    op = body.call(thisArg, _);
                }
                catch (e) {
                    op = [6, e];
                    y = 0;
                }
                finally {
                    f = t = 0;
                }
            if (op[0] & 5)
                throw op[1];
            return { value: op[0] ? op[1] : void 0, done: true };
        }
    }
    var __createBinding = Object.create ? (function (o, m, k, k2) {
        if (k2 === undefined)
            k2 = k;
        Object.defineProperty(o, k2, { enumerable: true, get: function () { return m[k]; } });
    }) : (function (o, m, k, k2) {
        if (k2 === undefined)
            k2 = k;
        o[k2] = m[k];
    });
    function __exportStar(m, o) {
        for (var p in m)
            if (p !== "default" && !Object.prototype.hasOwnProperty.call(o, p))
                __createBinding(o, m, p);
    }
    function __values(o) {
        var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
        if (m)
            return m.call(o);
        if (o && typeof o.length === "number")
            return {
                next: function () {
                    if (o && i >= o.length)
                        o = void 0;
                    return { value: o && o[i++], done: !o };
                }
            };
        throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }
    function __read(o, n) {
        var m = typeof Symbol === "function" && o[Symbol.iterator];
        if (!m)
            return o;
        var i = m.call(o), r, ar = [], e;
        try {
            while ((n === void 0 || n-- > 0) && !(r = i.next()).done)
                ar.push(r.value);
        }
        catch (error) {
            e = { error: error };
        }
        finally {
            try {
                if (r && !r.done && (m = i["return"]))
                    m.call(i);
            }
            finally {
                if (e)
                    throw e.error;
            }
        }
        return ar;
    }
    function __spread() {
        for (var ar = [], i = 0; i < arguments.length; i++)
            ar = ar.concat(__read(arguments[i]));
        return ar;
    }
    function __spreadArrays() {
        for (var s = 0, i = 0, il = arguments.length; i < il; i++)
            s += arguments[i].length;
        for (var r = Array(s), k = 0, i = 0; i < il; i++)
            for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
                r[k] = a[j];
        return r;
    }
    ;
    function __await(v) {
        return this instanceof __await ? (this.v = v, this) : new __await(v);
    }
    function __asyncGenerator(thisArg, _arguments, generator) {
        if (!Symbol.asyncIterator)
            throw new TypeError("Symbol.asyncIterator is not defined.");
        var g = generator.apply(thisArg, _arguments || []), i, q = [];
        return i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i;
        function verb(n) { if (g[n])
            i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; }
        function resume(n, v) { try {
            step(g[n](v));
        }
        catch (e) {
            settle(q[0][3], e);
        } }
        function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }
        function fulfill(value) { resume("next", value); }
        function reject(value) { resume("throw", value); }
        function settle(f, v) { if (f(v), q.shift(), q.length)
            resume(q[0][0], q[0][1]); }
    }
    function __asyncDelegator(o) {
        var i, p;
        return i = {}, verb("next"), verb("throw", function (e) { throw e; }), verb("return"), i[Symbol.iterator] = function () { return this; }, i;
        function verb(n, f) { i[n] = o[n] ? function (v) { return (p = !p) ? { value: __await(o[n](v)), done: n === "return" } : f ? f(v) : v; } : f; }
    }
    function __asyncValues(o) {
        if (!Symbol.asyncIterator)
            throw new TypeError("Symbol.asyncIterator is not defined.");
        var m = o[Symbol.asyncIterator], i;
        return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
        function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
        function settle(resolve, reject, d, v) { Promise.resolve(v).then(function (v) { resolve({ value: v, done: d }); }, reject); }
    }
    function __makeTemplateObject(cooked, raw) {
        if (Object.defineProperty) {
            Object.defineProperty(cooked, "raw", { value: raw });
        }
        else {
            cooked.raw = raw;
        }
        return cooked;
    }
    ;
    var __setModuleDefault = Object.create ? (function (o, v) {
        Object.defineProperty(o, "default", { enumerable: true, value: v });
    }) : function (o, v) {
        o["default"] = v;
    };
    function __importStar(mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k))
                    __createBinding(result, mod, k);
        __setModuleDefault(result, mod);
        return result;
    }
    function __importDefault(mod) {
        return (mod && mod.__esModule) ? mod : { default: mod };
    }
    function __classPrivateFieldGet(receiver, privateMap) {
        if (!privateMap.has(receiver)) {
            throw new TypeError("attempted to get private field on non-instance");
        }
        return privateMap.get(receiver);
    }
    function __classPrivateFieldSet(receiver, privateMap, value) {
        if (!privateMap.has(receiver)) {
            throw new TypeError("attempted to set private field on non-instance");
        }
        privateMap.set(receiver, value);
        return value;
    }

    function merge(object) {
        var e_1, _a;
        var sources = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            sources[_i - 1] = arguments[_i];
        }
        try {
            for (var sources_1 = __values(sources), sources_1_1 = sources_1.next(); !sources_1_1.done; sources_1_1 = sources_1.next()) {
                var source = sources_1_1.value;
                forEach(source, function (value, key) {
                    var myValue = object[key];
                    if (myValue instanceof Object) {
                        value = merge(clone(myValue), value);
                    }
                    object[key] = value;
                });
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (sources_1_1 && !sources_1_1.done && (_a = sources_1.return)) _a.call(sources_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return object;
    }

    /**
     * The opposite of `pick`; this method creates an object composed of the own enumerable string properties of object that are not omitted.
     *
     * Differences from lodash:
     * - `paths` must be direct keys of `object` (they cannot refer to deeper properties)
     * - does not work with arrays
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 15,567 bytes
     * - Micro-dash: 155 bytes
     */
    function omit(object) {
        var e_1, _a;
        var paths = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            paths[_i - 1] = arguments[_i];
        }
        var obj = clone(object) || {};
        try {
            for (var paths_1 = __values(paths), paths_1_1 = paths_1.next(); !paths_1_1.done; paths_1_1 = paths_1.next()) {
                var path = paths_1_1.value;
                delete obj[path];
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (paths_1_1 && !paths_1_1.done && (_a = paths_1.return)) _a.call(paths_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return obj;
    }

    function pickBy(object, predicate) {
        var obj = {};
        forOwn(object, function (item, key) {
            if (predicate(item, key)) {
                obj[key] = item;
            }
        });
        return obj;
    }

    function omitBy(object, predicate) {
        return pickBy(object, function (item, key) { return !predicate(item, key); });
    }

    /**
     * Creates an object composed of the picked `object` properties.
     *
     * Differences from lodash:
     * - `paths` must be direct properties of `object` (they cannot references deep properties)
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 7,718 bytes
     * - Micro-dash: 142 bytes
     */
    function pick(object) {
        var e_1, _a;
        var paths = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            paths[_i - 1] = arguments[_i];
        }
        var result = {};
        if (object != null) {
            try {
                for (var paths_1 = __values(paths), paths_1_1 = paths_1.next(); !paths_1_1.done; paths_1_1 = paths_1.next()) {
                    var path = paths_1_1.value;
                    result[path] = object[path];
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (paths_1_1 && !paths_1_1.done && (_a = paths_1.return)) _a.call(paths_1);
                }
                finally { if (e_1) throw e_1.error; }
            }
        }
        return result;
    }

    /**
     * This method is like `set()` except that it accepts `updater` to produce the value to set.
     *
     * **Note:** This method mutates `object`.
     *
     * Differences from lodash:
     * - only accepts an array for `path`, not a dot-separated string
     * - does not handle `customizer` returning `undefined`
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 5,842 bytes
     * - Micro-dash: 192 bytes
     */
    function update(object, path, updater) {
        if (object && path.length) {
            var current = object;
            var length = path.length;
            for (var i = 0; i < length; ++i) {
                var key = path[i];
                var value = current[key];
                if (i < length - 1) {
                    if (!(value instanceof Object)) {
                        value = Number.isInteger(path[i + 1]) ? [] : {};
                    }
                }
                else {
                    value = updater(value);
                }
                current = current[key] = value;
            }
        }
        return object;
    }

    /**
     * Sets the value at `path` of `object`. If a portion of `path` doesn't exist, it's created. Arrays are created for missing index properties while objects are created for all other missing properties.
     *
     * **Note:** This method mutates `object`.
     *
     * Differences from lodash:
     * - only accepts an array for `path`, not a dot-separated string
     * - does not handle `customizer` returning `undefined`
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 5,639 bytes
     * - Micro-dash: 192 bytes
     */
    function set(object, path, value) {
        return update(object, path, constant(value));
    }

    /**
     * Creates an array of own enumerable string keyed-value pairs for `object` which can be consumed by `fromPairs`.
     *
     * Differences from lodash:
     * - does not give any special consideration for arguments objects, strings, or prototype objects (e.g. many will have `'length'` in the returned array)
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 4,864 bytes
     * - Micro-dash: 185 bytes
     */
    function toPairs(object) {
        return keys(object).map(function (key) { return [key, object[key]]; });
    }

    function every(collection, predicate) {
        var result = true;
        forEach(collection, function (value, keyOrIndex) { return (result = !!predicate(value, keyOrIndex)); });
        return result;
    }

    function filter(collection, predicate) {
        var result = [];
        forEach(collection, function (item, indexOrKey) {
            if (predicate(item, indexOrKey)) {
                result.push(item);
            }
        });
        return result;
    }

    function find(collection, predicate, fromIndex) {
        var e_1, _a;
        if (fromIndex === void 0) { fromIndex = 0; }
        if (Array.isArray(collection)) {
            return collection
                .slice(fromIndex)
                .find(function (item, index) { return predicate(item, index); });
        }
        else {
            try {
                for (var _b = __values(keysOfNonArray(collection).slice(fromIndex)), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var key = _c.value;
                    var item = collection[key];
                    if (predicate(item, key)) {
                        return item;
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                }
                finally { if (e_1) throw e_1.error; }
            }
        }
    }

    function map(collection, iteratee) {
        var mapped = [];
        forEach(collection, function (value, keyOrIndex) {
            mapped.push(iteratee(value, keyOrIndex));
        });
        return mapped;
    }

    function flatMap(collection, iteratee) {
        return flatten(map(collection, iteratee));
    }

    /**
     * Creates an object composed of keys generated from the results of running each element of `collection` thru `iteratee`. The order of grouped values is determined by the order they occur in `collection`. The corresponding value of each key is an array of elements responsible for generating the key.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 14,277 bytes
     * - Micro-dash: 331 bytes
     */
    function groupBy(collection, iteratee) {
        return transform(collection, function (accumulator, value) {
            var key = iteratee(value);
            var group = accumulator[key];
            if (!Array.isArray(group)) {
                accumulator[key] = group = [];
            }
            group.push(value);
        }, {});
    }

    /**
     * Checks if `value` is in `collection`. If `collection` is a string, it's checked for a substring of `value`. If `fromIndex` is negative, it's used as the offset from the end of `collection`.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 4,546 bytes
     * - Micro-dash: 319 bytes
     */
    function includes(collection, value, fromIndex) {
        if (fromIndex === void 0) { fromIndex = 0; }
        if (isString(collection)) {
            if (fromIndex > collection.length) {
                return false;
            }
        }
        else {
            collection = toArray(collection);
        }
        return collection.includes(value, fromIndex);
    }

    function keyBy(collection, iteratee) {
        var obj = {};
        forEach(collection, function (value) {
            obj[iteratee(value)] = value;
        });
        return obj;
    }

    /** @hidden */
    function doReduce(iterationFn, collection, iteratee, accumulator, initAccum) {
        iterationFn(collection, function (value, indexOrKey) {
            if (initAccum) {
                accumulator = value;
                initAccum = false;
            }
            else {
                accumulator = iteratee(accumulator, value, indexOrKey);
            }
        });
        return accumulator;
    }

    function reduce(collection, iteratee, accumulator) {
        return doReduce(forEach, collection, iteratee, accumulator, arguments.length < 3);
    }

    function reduceRight(collection, iteratee, accumulator) {
        return doReduce(forEachRight, collection, iteratee, accumulator, arguments.length < 3);
    }

    /** @hidden */
    function findExtreme(array, iteratee, shouldReplace) {
        var current;
        var currentCriterion;
        forEachOfArray(array, function (candidate) {
            var candidateCriterion = iteratee(candidate);
            if (current === undefined ||
                shouldReplace(candidateCriterion, currentCriterion)) {
                current = candidate;
                currentCriterion = candidateCriterion;
            }
        });
        // tslint:disable-next-line:no-non-null-assertion
        return current;
    }

    /**
     * This method is like `max` except that it accepts `iteratee` which is invoked for each element in `array` to generate the criterion by which the value is ranked.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 8,302 bytes
     * - Micro-dash: 165 bytes
     */
    function maxBy(array, iteratee) {
        return findExtreme(array, iteratee, function (candidate, current) { return candidate > current; });
    }

    /**
     * This method is like `min` except that it accepts `iteratee` which is invoked for each element in `array` to generate the criterion by which the value is ranked.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 8,302 bytes
     * - Micro-dash: 165 bytes
     */
    function minBy(array, iteratee) {
        return findExtreme(array, iteratee, function (candidate, current) { return candidate < current; });
    }

    function random() {
        var _a, _b, _c;
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var lower = 0;
        var upper = 1;
        if (isFinite(args[1])) {
            _a = __read(args, 2), lower = _a[0], upper = _a[1];
        }
        else if (isFinite(args[0])) {
            _b = __read(args, 1), upper = _b[0];
        }
        _c = __read([lower, upper].sort(), 2), lower = _c[0], upper = _c[1];
        var floating = last(args);
        if (!isBoolean(floating)) {
            floating = !Number.isInteger(lower) && !Number.isInteger(upper);
        }
        return randomInt(lower, upper, floating);
    }
    var randomInt = function (lower, upper, floating) {
        var range = upper - lower;
        if (!floating) {
            ++range;
        }
        var result = Math.random() * range + lower;
        if (!floating) {
            result = Math.floor(result);
        }
        return result;
    };
    // /**
    //  * Produces a random number between the inclusive `lower` and `upper` bounds. If only one argument is provided a number between `0` and the given number is returned. If `floating` is true, or either `lower` or `upper` are floats, a floating-point number is returned instead of an integer.
    //  *
    //  * **Note:** JavaScript follows the IEEE-754 standard for resolving floating-point values which can produce unexpected results.
    //  *
    //  * Differences from lodash:
    //  * - requires each argument to be provided in its documented spot. E.g. `floating` must always be the third argument, and `lower` should not be greater than `upper`.
    //  * - does not coerce arguments into finite numbers. E.g. if you supply `NaN` or `Infinity`, results are not guaranteed.
    //  *
    //  * Contribution to minified bundle size, when it is the only function imported:
    //  */
    // export function random(
    //   lower = 0,
    //   upper = 1,
    //   floating = !(Number.isInteger(lower) && Number.isInteger(upper)),
    // ) {
    //   let range = upper - lower;
    //   if (!floating) {
    //     ++range;
    //   }
    //   let result = Math.random() * range + lower;
    //   if (!floating) {
    //     result = Math.floor(result);
    //   }
    //   return result;
    // }

    /**
     * Computes `number` rounded to `precision`.
     *
     * ```ts
     * round(4.006); // 4
     * round(4.006, 2); // 4.01
     * round(4060, -2); // 4100
     * ```
     *
     * Differences from lodash:
     * - may return `NaN` with large `precision` values
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 1,826 bytes
     * - Micro-dash: 111 bytes
     */
    // tslint:disable-next-line:variable-name
    function round(number, precision) {
        if (precision === void 0) { precision = 0; }
        var factor = Math.pow(10, Math.trunc(precision));
        return Math.round(number * factor) / factor;
    }

    /**
     * This method is like `sum` except that it accepts `iteratee` which is invoked for each element in `array` to generate the value to be summed. The iteratee is invoked with one argument: (value).
     *
     * Differences from lodash:
     * - does not skip `undefined` values
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 8,252 bytes
     * - Micro-dash: 42 bytes
     */
    function sumBy(array, iteratee) {
        return array.reduce(function (sum, element) { return sum + iteratee(element); }, 0);
    }

    /**
     * Gets a random element from `collection`.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 3,654 bytes
     * - Micro-dash: 459 bytes
     */
    function sample(collection) {
        collection = toArray(collection);
        return collection[random(collection.length - 1)];
    }

    function sampleSize(collection, n) {
        if (n === void 0) { n = 1; }
        var values = map(collection, identity);
        return times(Math.min(n, values.length), function () { return pullAt(values, randomInt(0, values.length - 1, false))[0]; });
    }

    function size(collection) {
        return keys(collection).length;
    }

    function shuffle(collection) {
        return sampleSize(collection, size(collection));
    }

    function some(collection, predicate) {
        var none = true;
        forEach(collection, function (item, keyOrIndex) { return (none = !predicate(item, keyOrIndex)); });
        return !none;
    }

    /**
     * Creates an array of elements, sorted in ascending order by the results of running each element in a collection thru each iteratee. This method performs a stable sort, that is, it preserves the original sort order of equal elements.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 15,948 bytes
     * - Micro-dash: 641 bytes
     */
    function sortBy(collection, iteratees) {
        var fns = castArray(iteratees);
        var metas = map(collection, function (value) { return Object.assign(fns.map(function (fn) { return fn(value); }), { value: value }); });
        metas.sort(function (m1, m2) {
            for (var i = 0; i < m1.length; ++i) {
                var comp = compareValues(m1[i], m2[i]);
                if (comp) {
                    return comp;
                }
            }
        });
        return metas.map(function (e) { return e.value; });
    }
    /** @hidden */
    var compareValues = function (x, y) {
        var v1 = getSortOrdinal(x);
        var v2 = getSortOrdinal(y);
        if (v1 === v2) {
            v1 = x;
            v2 = y;
        }
        if (v1 < v2) {
            return -1;
        }
        else if (v1 > v2) {
            return 1;
        }
    };
    /** @hidden */
    var getSortOrdinal = function (value) {
        if (Number.isNaN(value)) {
            return 3;
        }
        if (value === undefined) {
            return 2;
        }
        if (value === null) {
            return 1;
        }
        return 0;
    };
    var ɵ0 = getSortOrdinal;

    function transform(object, iteratee, accumulator) {
        if (accumulator === void 0) { accumulator = {}; }
        forEach(object, function (value, key) { return iteratee(accumulator, value, key); });
        return accumulator;
    }

    /**
     * Creates an array of the own enumerable string keyed property values of `object`.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 3,421 bytes
     * - Micro-dash: 164 bytes
     */
    function values(object) {
        return keys(object).map(function (key) { return object[key]; });
    }
    /** @hidden */
    function valuesOfNonArray(object) {
        return keysOfNonArray(object).map(function (key) { return object[key]; });
    }

    /**
     * This method is like `clone` except that it recursively clones `value`.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 12,221 bytes
     * - Micro-dash: 405 bytes
     */
    function cloneDeep(value) {
        if (value instanceof Object) {
            value = clone(value);
            forOwn(value, function (item, key) {
                value[key] = cloneDeep(item);
            });
        }
        return value;
    }

    /**
     * Checks if `value` is classified as a boolean primitive.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 693 bytes
     * - Micro-dash: 56 bytes
     */
    function isBoolean(value) {
        return value === !!value;
    }

    /**
     * Checks if `value` is an empty object or collection.
     *
     * Objects are considered empty if they have no own enumerable string keyed properties.
     *
     * Arrays are considered empty if they have a `length` of `0`.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 4,302 bytes
     * - Micro-dash: 115 bytes
     */
    function isEmpty(value) {
        if (!Array.isArray(value)) {
            value = keysOfNonArray(value);
        }
        return value.length === 0;
    }

    /**
     * Performs a deep comparison between two values to determine if they are equivalent.
     *
     * Differences from lodash:
     * - cannot handle circular references
     * - does not give special treatment to arrays; their own properties are compared just like other objects. Note this means sparse arrays are not equal to their dense "equivalents".
     * - `isEqual(0, -0) === false`
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 10,713 bytes
     * - Micro-dash: 294 bytes
     */
    function isEqual(value, other) {
        var e_1, _a, e_2, _b;
        if (Object.is(value, other)) {
            // covers e.g. NaN === NaN
            return true;
        }
        if (!(value instanceof Object && other instanceof Object)) {
            return false;
        }
        try {
            for (var _c = __values(keysOfNonArray(value)), _d = _c.next(); !_d.done; _d = _c.next()) {
                var key = _d.value;
                if (!other.hasOwnProperty(key)) {
                    return false;
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_d && !_d.done && (_a = _c.return)) _a.call(_c);
            }
            finally { if (e_1) throw e_1.error; }
        }
        try {
            for (var _e = __values(keysOfNonArray(other)), _f = _e.next(); !_f.done; _f = _e.next()) {
                var key = _f.value;
                if (!isEqual(value[key], other[key])) {
                    return false;
                }
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_f && !_f.done && (_b = _e.return)) _b.call(_e);
            }
            finally { if (e_2) throw e_2.error; }
        }
        return true;
    }

    /**
     * Checks if `value` is classified as a `Function` object.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 824 bytes
     * - Micro-dash: 86 bytes
     */
    function isFunction(value) {
        return typeof value === 'function';
    }

    /**
     * Performs a partial deep comparison between `object` and `source` to determine if `object` contains equivalent property values.
     *
     * *Note:* This method is equivalent to `matches` when source is partially applied.
     *
     * Partial comparisons will match empty array and empty object source values against any array or object value, respectively.
     *
     * Differences from lodash:
     * - does not match `0` to `-0`
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 11,113 bytes
     * - Micro-dash: 662 bytes
     */
    function isMatch(object, source) {
        return isEmpty(source) || isMatch0(object, source);
    }
    /** @hidden */
    function isMatch0(object, source) {
        var e_1, _a;
        if (Object.is(object, source)) {
            return true;
        }
        if (!(object instanceof Object && source instanceof Object)) {
            return false;
        }
        var isArray = Array.isArray(object);
        if (isArray !== Array.isArray(source)) {
            return false;
        }
        if (isArray) {
            var unmatched = object.slice();
            var _loop_1 = function (innerSource) {
                var i = unmatched.findIndex(function (item) { return isMatch0(item, innerSource); });
                if (i < 0) {
                    return { value: false };
                }
                unmatched.splice(i, 1);
            };
            try {
                for (var source_1 = __values(source), source_1_1 = source_1.next(); !source_1_1.done; source_1_1 = source_1.next()) {
                    var innerSource = source_1_1.value;
                    var state_1 = _loop_1(innerSource);
                    if (typeof state_1 === "object")
                        return state_1.value;
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (source_1_1 && !source_1_1.done && (_a = source_1.return)) _a.call(source_1);
                }
                finally { if (e_1) throw e_1.error; }
            }
            return true;
        }
        else {
            return every(source, function (value, key) { return object.hasOwnProperty(key) && isMatch0(object[key], value); });
        }
    }

    /**
     * Checks if value is `null` or `undefined`.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 78 bytes
     * - Micro-dash: 73 bytes
     */
    function isNil(value) {
        return value == null;
    }

    /**
     * Checks if `value` is classified as a `Number` primitive.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 695 bytes
     * - Micro-dash: 65 bytes
     */
    function isNumber(value) {
        return typeof value === 'number';
    }

    /**
     * Checks if `value` is the [language type](http://www.ecma-international.org/ecma-262/7.0/#sec-ecmascript-language-types) of `Object`. _(e.g. arrays, functions, objects, regexes, `new Number(0)`, and `new String('')`)_
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 105 bytes
     * - Micro-dash: 67 bytes
     */
    function isObject(value) {
        return value instanceof Object;
    }

    /**
     * Checks if `value` is classified as a `RegExp` object.
     *
     * Differences from lodash:
     * - returns `true` for subclasses of `RegExp`
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 1,039 bytes
     * - Micro-dash: 83 bytes
     */
    function isRegExp(value) {
        return value instanceof RegExp;
    }

    /**
     * Checks if `value` is classified as a String primitive.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 752 bytes
     * - Micro-dash: 99 bytes
     */
    function isString(value) {
        return typeof value === 'string';
    }

    /**
     * Checks if `value` is `undefined`.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 81 bytes
     * - Micro-dash: 76 bytes
     */
    function isUndefined(value) {
        return value === undefined;
    }

    function toArray(value) {
        if (value && value[Symbol.iterator]) {
            return Array.from(value);
        }
        else {
            return valuesOfNonArray(value);
        }
    }

    /**
     * Converts `value` to a string. An empty string is returned for `null` and `undefined` values.
     *
     * Differences:
     * - `null` and `undefined` array references are blank
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 1,049 bytes
     * - Micro-dash: 84 bytes
     */
    function toString(value) {
        return [value].toString();
    }

    /**
     * Creates a function that performs a partial deep comparison between a given `object` and `source`, returning `true` if the given object has equivalent property values, else `false`.
     *
     * Note: The created function is equivalent to `isMatch` with source partially applied.
     *
     * Partial comparisons will match empty array and empty object source values against any array or object value, respectively.
     *
     * Differences from lodash:
     * - does not match `0` to `-0`
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 15,801 bytes
     * - Micro-dash: 975 bytes
     */
    function matches(source) {
        source = cloneDeep(source);
        return function (value) { return isMatch(value, source); };
    }

    /**
     * This method returns undefined.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 47 bytes
     * - Micro-dash: 42 bytes
     */
    function noop() { }

    /**
     * Invokes the iteratee `n` times, returning an array of the results of each invocation.
     *
     * Differences from lodash:
     * - has undefined behavior when given a non natural number for `n`
     * - does not provide a default for `iteratee`
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 1,483 bytes
     * - Micro-dash: 68 bytes
     */
    function times(n, iteratee) {
        var result = [];
        for (var i = 0; i < n; ++i) {
            result[i] = iteratee(i);
        }
        return result;
    }

    function range(start, end, step) {
        if (isUndefined(end)) {
            end = start;
            start = 0;
        }
        if (isUndefined(step)) {
            step = end < start ? -1 : 1;
        }
        return times(Math.abs((end - start) / (step || 1)), function (i) { return start + step * i; });
    }

    /** @hidden */
    var nextId = 1;
    /**
     * Generates a unique ID. If `prefix` is given, the ID is appended to it.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 1,062 bytes
     * - Micro-dash: 77 bytes
     */
    function uniqueId(prefix) {
        if (prefix === void 0) { prefix = ''; }
        return prefix + nextId++;
    }

    /**
     * Creates an array with all falsey values removed. The values `false`, `null`, `0`, `""`, `undefined`, and `NaN` are falsey.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 96 bytes
     * - Micro-dash: 35 bytes
     */
    function compact(array) {
        return array.filter(identity);
    }

    /**
     * Creates a new array concatenating `array` with any additional arrays and/or values.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 1,410 bytes
     * - Micro-dash: 36 bytes
     */
    function concat(array) {
        var values = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            values[_i - 1] = arguments[_i];
        }
        return array.concat.apply(array, __spread(values));
    }

    function remove(array, predicate) {
        var removed = [];
        for (var i = array.length; --i >= 0;) {
            if (predicate(array[i], i)) {
                removed.unshift(array[i]);
                array.splice(i, 1);
            }
        }
        return removed;
    }

    /**
     * This method is like `pull` except that it accepts an array of values to remove.
     *
     * **Note:** Unlike `difference`, this method mutates array.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 815 bytes
     * - Micro-dash: 199 bytes
     */
    function pullAll(array, values) {
        var e_1, _a;
        var _loop_1 = function (value) {
            remove(array, function (item) { return Object.is(item, value); });
        };
        try {
            for (var _b = __values(values.slice()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var value = _c.value;
                _loop_1(value);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return array;
    }
    // TODO: return type could be narrower?

    /**
     * Flattens `array` a single level deep.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 1,183 bytes
     * - Micro-dash: 29 bytes
     */
    function flatten(array) {
        return [].concat.apply([], __spread(array));
    }

    /**
     * Creates an array of array values not included in the other given arrays. The order and references of result values are determined by the first array.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 6,275 bytes
     * - Micro-dash: 278 bytes
     */
    function difference(array) {
        var values = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            values[_i - 1] = arguments[_i];
        }
        return pullAll(array.slice(), flatten(values));
    }

    /**
     * Gets all but the last element of `array`.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 207 bytes
     * - Micro-dash: 33 bytes
     */
    function initial(array) {
        return array.slice(0, -1);
    }

    /**
     * Gets the last element of array.
     *
     * Differences from lodash:
     * - no special consideration is given to string-keyed values set on the array
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 68 bytes
     * - Micro-dash: 43 bytes
     */
    function last(array) {
        return array[array.length - 1];
    }

    function nth(array, index) {
        if (array) {
            return array[index < 0 ? array.length + index : index];
        }
    }

    /**
     * Removes all given values from array using `SameValueZero` for equality comparisons.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 2,984 bytes
     * - Micro-dash: 177 bytes
     */
    function pull(array) {
        var values = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            values[_i - 1] = arguments[_i];
        }
        return pullAll(array, values);
    }

    /**
     * Removes elements from array corresponding to indexes and returns an array of removed elements.
     *
     * *Note:* This method mutates `array`.
     *
     * Differences from lodash:
     * - behavior is undefined when attempting to pull attributes keyed with anything other than positive integers
     * - does not support deep paths
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 7,703 bytes
     * - Micro-dash: 216 bytes
     */
    function pullAt(array) {
        var e_1, _a;
        var indexes = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            indexes[_i - 1] = arguments[_i];
        }
        var flattenedIndexes = flatten(indexes);
        var result = flattenedIndexes.map(function (i) { return array[i]; });
        var lastI;
        try {
            for (var _b = __values(flattenedIndexes.sort(function (a, b) { return b - a; })), _c = _b.next(); !_c.done; _c = _b.next()) {
                var i = _c.value;
                if (i !== lastI) {
                    array.splice(i, 1);
                }
                lastI = i;
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return result;
    }

    /**
     * Uses a binary search to determine the lowest index at which `value` should be inserted into `array` in order to maintain its sort order.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 1,266 bytes
     * - Micro-dash: 277 bytes
     */
    function sortedIndex(array, value) {
        var min = 0;
        var max = array ? array.length - 1 : 0;
        while (max > min) {
            var mid = Math.floor((max + min) / 2);
            if (compareValues(array[mid], value) < 0) {
                min = mid + 1;
            }
            else {
                max = mid;
            }
        }
        return min;
    }

    /**
     * Creates an array of unique values, in order, from all given `arrays`.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 6,214 bytes
     * - Micro-dash: 72 bytes
     */
    function union() {
        var arrays = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            arrays[_i] = arguments[_i];
        }
        return Array.from(new Set(flatten(arrays)));
    }

    /**
     * Creates a duplicate-free version of an array, using `SameValueZero` for equality comparisons, in which only the first occurrence of each element is kept. The order of result values is determined by the order they occur in the array.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 4,573 bytes
     * - Micro-dash: 40 bytes
     */
    function uniq(array) {
        return Array.from(new Set(array));
    }

    /**
     * This method is like `_.uniq` except that it accepts `iteratee` which is invoked for each element in `array` to generate the criterion by which uniqueness is computed. The order of result values is determined by the order they occur in the array.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 8,958 bytes
     * - Micro-dash: 115 bytes
     */
    function uniqBy(array, iteratee) {
        var seen = new Set();
        return array.filter(function (element) {
            var key = iteratee(element);
            var isNew = !seen.has(key);
            seen.add(key);
            return isNew;
        });
    }

    /**
     * Creates an array excluding all given values.
     *
     * Differences from lodash:
     * - Uses triple equals rather than `SameValueZero`.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 5,650 bytes
     * - Micro-dash: 64 bytes
     */
    function without(array) {
        var values = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            values[_i - 1] = arguments[_i];
        }
        return array.filter(function (item) { return values.indexOf(item) === -1; });
    }

    function zip() {
        var arrays = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            arrays[_i] = arguments[_i];
        }
        var length = Math.max.apply(Math, __spread([0], arrays.map(function (a) { return (a ? a.length : 0); })));
        return times(length, function (i) { return arrays.map(function (a) { return (a ? a[i] : undefined); }); });
    }

    function zipObject(props, values) {
        return transform(props, function (accumulator, prop, index) {
            accumulator[prop] = values[index];
        });
    }

    function bindKey(object, key) {
        var partials = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            partials[_i - 2] = arguments[_i];
        }
        return function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            return object[key].apply(object, __spread(partials, args));
        };
    }

    function partial(func) {
        var partials = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            partials[_i - 1] = arguments[_i];
        }
        return func.bind.apply(func, __spread([undefined], partials));
    }

    function curry(func, arity) {
        if (arity === void 0) { arity = func.length; }
        return function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            if (args.length < arity) {
                return curry(partial.apply(void 0, __spread([func], args)), (arity - args.length));
            }
            else {
                return func.apply(this, args);
            }
        };
    }

    /**
     * Creates a debounced function that delays invoking `func` until after `wait` milliseconds have elapsed since the last time the debounced function was invoked. The debounced function comes with a `cancel` method to cancel delayed `func` invocations. The `func` is invoked with the last arguments provided to the debounced function.
     *
     * If `wait` is 0, `func` invocation is deferred until to the next tick, similar to `setTimeout` with a timeout of `0`.
     *
     * Differences from lodash:
     * - the debounced function does not come with a `flush` method
     * - does not accept options
     * - does not return the results of the last invocation
     * - does not make any guarantees about the value of `this` in `func`
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 2,064 bytes
     * - Micro-dash: 166 bytes
     */
    function debounce(func, wait) {
        if (wait === void 0) { wait = 0; }
        var timeoutId;
        var cancel = function () {
            clearTimeout(timeoutId);
        };
        var debounced = function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            cancel();
            timeoutId = setTimeout.apply(void 0, __spread([func, wait], args));
        };
        return Object.assign(debounced, { cancel: cancel });
    }

    /**
     * Creates a function that memoizes the result of `func`. If `resolver` is provided, it determines the cache key for storing the result based on the arguments provided to the memoized function. By default, the first argument provided to the memoized function is used as the map cache key. The `func` is invoked with the `this` binding of the memoized function.
     *
     * **Note:** The cache is exposed as the `cache` property on the memoized function.
     *
     * Differences from lodash:
     * - does not coerce keys to a string; any object can be used as in an ES6 `Map`
     * - does not let you customize cache creation
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 3,852 bytes
     * - Micro-dash: 243 bytes
     *
     * @param resolver The function to resolve the cache key.
     */
    function memoize(func, resolver) {
        if (resolver === void 0) { resolver = identity; }
        var memoized = function () {
            var cache = memoized.cache;
            var key = resolver.apply(this, arguments);
            if (cache.has(key)) {
                return cache.get(key);
            }
            else {
                var value = func.apply(this, arguments);
                cache.set(key, value);
                return value;
            }
        };
        memoized.cache = new Map();
        return memoized;
    }

    /**
     * Creates a function that is restricted to invoking `func` once. Repeat calls to the function return the value of the first invocation. The `func` is invoked with the `this` binding and arguments of the created function.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 1,423 bytes
     * - Micro-dash: 99 bytes
     */
    function once(func) {
        var result;
        var needsCall = true;
        return function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            if (needsCall) {
                needsCall = false;
                result = func.apply(this, args);
                // allow func and any of its variables to be garbage collected
                func = 0;
            }
            return result;
        };
    }

    /**
     * Creates a throttled function that only invokes `func` at most once per every `wait` milliseconds. The throttled function comes with a `cancel` method to cancel delayed `func` invocations and a `flush` method to immediately invoke them. Provide `options` to indicate whether `func` should be invoked on the leading and/or trailing edge of the `wait` timeout. The `func` is invoked with the last arguments provided to the throttled function.
     *
     *  **Note**: If `leading` and `trailing` options are `true`, `func` is invoked on the trailing edge of the timeout only if the throttled function is invoked more than once during the `wait` timeout.
     *
     * If `wait` is 0 and `leading` is `false`, `func` invocation is deferred until to the next tick, similar to `setTimeout` with a timeout of 0.
     *
     * Differences from lodash:
     * - does not return the results of the last invocation
     * - does not make any guarantees about the value of `this` in `func`
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 2,290 bytes
     * - Micro-dash: 394 bytes
     */
    function throttle(func, wait, _a) {
        if (wait === void 0) { wait = 0; }
        var _b = _a === void 0 ? {} : _a, _c = _b.leading, leading = _c === void 0 ? true : _c, _d = _b.trailing, trailing = _d === void 0 ? true : _d;
        var tail = 0;
        var nextArgs;
        var timeoutId;
        // helpers to save some bytes
        var now = function () { return performance.now(); };
        var setNewTail = function () {
            tail = now() + wait;
        };
        var cancel = function () {
            clearTimeout(timeoutId);
            timeoutId = undefined;
            nextArgs = undefined;
            tail = 0;
        };
        var flush = function () {
            if (nextArgs) {
                var args = nextArgs;
                cancel();
                setNewTail();
                func.apply(void 0, __spread(args));
            }
        };
        var throttled = function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            nextArgs = args;
            var delay = Math.max(0, tail - now());
            if (!delay && (leading || timeoutId)) {
                flush();
            }
            else if (trailing) {
                if (!delay) {
                    setNewTail();
                }
                clearTimeout(timeoutId);
                timeoutId = setTimeout(flush, delay || wait);
            }
        };
        return Object.assign(throttled, { cancel: cancel, flush: flush });
    }

    /**
     * Clamps `number` within the inclusive lower and upper bounds.
     *
     * Differences from lodash:
     * - `lower` is required
     * - does not coerce bounds that are `NaN` to be `0`
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 1,367 bytes
     * - Micro-dash: 119 bytes
     */
    // tslint:disable-next-line:variable-name
    function clamp(number, lower, upper) {
        return Math.min(upper, Math.max(lower, number));
    }

    /**
     * Checks if `n` is between `start` and up to, but not including, `end`. If start is greater than end the params are swapped to support negative ranges.
     *
     * Differences from lodash:
     * - `start` and `end` are both required
     * - does not coerce falsey bounds to `0`
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 1,396 bytes
     * - Micro-dash: 127 bytes
     */
    // tslint:disable-next-line:variable-name
    function inRange(number, start, end) {
        return number >= Math.min(start, end) && number < Math.max(start, end);
    }

    /**
     * Converts the first character of `string` to upper case.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 1,940 bytes
     * - Micro-dash: 102 bytes
     */
    // tslint:disable-next-line:variable-name
    function upperFirst(string) {
        return string ? string.charAt(0).toUpperCase() + string.slice(1) : '';
    }

    /**
     * Converts the first character of `string` to upper case and the remaining to lower case.
     *
     * Differences from lodash:
     * - requires `string` to be a string
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 1,937 bytes
     * - Micro-dash: 76 bytes
     */
    // tslint:disable-next-line:variable-name
    function capitalize(string) {
        return upperFirst(string.toLowerCase());
    }

    /**
     * Splits `string` into an array of its words.
     *
     * Differences from lodash:
     * - does not accept a `pattern` argument
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 2,283 bytes
     * - Micro-dash: 187 bytes
     */
    // tslint:disable-next-line:variable-name
    function words(string) {
        return (string
            // between lower & upper
            .replace(/([a-z])([A-Z])/g, '$1 $2')
            // between number & letter
            .replace(/(\d)([A-Za-z])/g, '$1 $2')
            .replace(/([A-Za-z])(\d)/g, '$1 $2')
            // before last upper in a sequence followed by lower
            .replace(/\b([A-Z]+)([A-Z])([a-z])/, '$1 $2$3')
            .match(/[A-Za-z0-9]+/g) || []);
    }

    /**
     * Converts `string` to camel case.
     *
     * Differences from lodash:
     * - requires `string` to be a string
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 6,097 bytes
     * - Micro-dash: 344 bytes
     */
    // tslint:disable-next-line:variable-name
    function camelCase(string) {
        return words(string)
            .map(function (w, i) { return (i ? capitalize(w) : w.toLowerCase()); })
            .join('');
    }

    /**
     * Converts `string`, as a whole, to lower case just like `String#toLowerCase`.
     *
     * Differences from lodash:
     * - requires `string` to be a string
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 996 bytes
     * - Micro-dash: 27 bytes
     */
    // tslint:disable-next-line:variable-name
    function toLower(string) {
        return string.toLowerCase();
    }

    /**
     * Converts `string` to [kebab case](https://en.wikipedia.org/wiki/Letter_case#Special_case_styles).
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 5,154 bytes
     * - Micro-dash: 254 bytes
     */
    // tslint:disable-next-line:variable-name
    function kebabCase(string) {
        return words(string).map(toLower).join('-');
    }

    /**
     * Converts the first character of `string` to lower case.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 1,940 bytes
     * - Micro-dash: 102 bytes
     */
    // tslint:disable-next-line:variable-name
    function lowerFirst(string) {
        return string ? string.charAt(0).toLowerCase() + string.slice(1) : '';
    }

    /**
     * Pads `string` on the left and right sides if it's shorter than `length`. Padding characters are truncated if they can't be evenly divided by `length`.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 3,337 bytes
     * - Micro-dash: 217 bytes
     */
    function pad(s, length, chars) {
        if (chars === void 0) { chars = ' '; }
        var needed = (length - s.length) / 2;
        return (_makePadding(Math.floor(needed), chars) +
            s +
            _makePadding(Math.ceil(needed), chars));
    }
    /** @hidden */
    function makePadding(s, length, chars) {
        return _makePadding(length - s.length, chars);
    }
    /** @hidden */
    function _makePadding(needed, chars) {
        return needed > 0 && chars.length > 0
            ? chars.repeat(Math.ceil(needed / chars.length)).substr(0, needed)
            : '';
    }

    /**
     * Pads `string` on the left side if it's shorter than `length`. Padding characters are truncated if they exceed `length`.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 3,286 bytes
     * - Micro-dash: 201 bytes
     */
    function padStart(s, length, chars) {
        if (chars === void 0) { chars = ' '; }
        return makePadding(s, length, chars) + s;
    }

    /**
     * Pads `string` on the right side if it's shorter than `length`. Padding characters are truncated if they exceed `length`.
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 3,286 bytes
     * - Micro-dash: 201 bytes
     */
    function padEnd(s, length, chars) {
        if (chars === void 0) { chars = ' '; }
        return s + makePadding(s, length, chars);
    }

    /**
     * Repeats the given string `n` times.
     *
     * Differences from lodash:
     * - does not work as an iteratee for methods like `map`
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 2,344 bytes
     * - Micro-dash: 96 bytes
     */
    // tslint:disable-next-line:variable-name
    function repeat(string, n) {
        // tslint:disable-next-line:no-bitwise
        return n < 0 ? '' : new Array(n | 0).fill(string).join('');
    }

    /**
     * Converts `string` to [snake case](https://en.wikipedia.org/wiki/Snake_case)
     *
     * Contribution to minified bundle size, when it is the only function imported:
     * - Lodash: 5,171 bytes
     * - Micro-dash: 268 bytes
     */
    // tslint:disable-next-line:variable-name
    function snakeCase(string) {
        return words(string).map(toLower).join('_');
    }

    /*
     * Public API Surface of micro-dash
     */

    /**
     * Generated bundle index. Do not edit.
     */

    exports.bindKey = bindKey;
    exports.camelCase = camelCase;
    exports.capitalize = capitalize;
    exports.castArray = castArray;
    exports.chunk = chunk;
    exports.clamp = clamp;
    exports.clone = clone;
    exports.cloneDeep = cloneDeep;
    exports.compact = compact;
    exports.concat = concat;
    exports.constant = constant;
    exports.curry = curry;
    exports.debounce = debounce;
    exports.difference = difference;
    exports.every = every;
    exports.filter = filter;
    exports.find = find;
    exports.findKey = findKey;
    exports.flatMap = flatMap;
    exports.flatten = flatten;
    exports.flow = flow;
    exports.flowRight = flowRight;
    exports.forEach = forEach;
    exports.forEachRight = forEachRight;
    exports.forOwn = forOwn;
    exports.forOwnRight = forOwnRight;
    exports.functions = functions;
    exports.get = get;
    exports.groupBy = groupBy;
    exports.identity = identity;
    exports.inRange = inRange;
    exports.includes = includes;
    exports.initial = initial;
    exports.invoke = invoke;
    exports.isBoolean = isBoolean;
    exports.isEmpty = isEmpty;
    exports.isEqual = isEqual;
    exports.isFunction = isFunction;
    exports.isMatch = isMatch;
    exports.isNil = isNil;
    exports.isNumber = isNumber;
    exports.isObject = isObject;
    exports.isRegExp = isRegExp;
    exports.isString = isString;
    exports.isUndefined = isUndefined;
    exports.kebabCase = kebabCase;
    exports.keyBy = keyBy;
    exports.keys = keys;
    exports.last = last;
    exports.lowerFirst = lowerFirst;
    exports.map = map;
    exports.mapValues = mapValues;
    exports.matches = matches;
    exports.maxBy = maxBy;
    exports.memoize = memoize;
    exports.merge = merge;
    exports.minBy = minBy;
    exports.noop = noop;
    exports.nth = nth;
    exports.omit = omit;
    exports.omitBy = omitBy;
    exports.once = once;
    exports.pad = pad;
    exports.padEnd = padEnd;
    exports.padStart = padStart;
    exports.partial = partial;
    exports.pick = pick;
    exports.pickBy = pickBy;
    exports.pull = pull;
    exports.pullAll = pullAll;
    exports.pullAt = pullAt;
    exports.random = random;
    exports.range = range;
    exports.reduce = reduce;
    exports.reduceRight = reduceRight;
    exports.remove = remove;
    exports.repeat = repeat;
    exports.round = round;
    exports.sample = sample;
    exports.sampleSize = sampleSize;
    exports.set = set;
    exports.shuffle = shuffle;
    exports.size = size;
    exports.snakeCase = snakeCase;
    exports.some = some;
    exports.sortBy = sortBy;
    exports.sortedIndex = sortedIndex;
    exports.sumBy = sumBy;
    exports.throttle = throttle;
    exports.times = times;
    exports.toArray = toArray;
    exports.toLower = toLower;
    exports.toPairs = toPairs;
    exports.toString = toString;
    exports.transform = transform;
    exports.union = union;
    exports.uniq = uniq;
    exports.uniqBy = uniqBy;
    exports.uniqueId = uniqueId;
    exports.update = update;
    exports.upperFirst = upperFirst;
    exports.values = values;
    exports.without = without;
    exports.words = words;
    exports.zip = zip;
    exports.zipObject = zipObject;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=micro-dash.umd.js.map
