import { DeepPartial } from 'utility-types';
/**
 * Performs a partial deep comparison between `object` and `source` to determine if `object` contains equivalent property values.
 *
 * *Note:* This method is equivalent to `matches` when source is partially applied.
 *
 * Partial comparisons will match empty array and empty object source values against any array or object value, respectively.
 *
 * Differences from lodash:
 * - does not match `0` to `-0`
 *
 * Contribution to minified bundle size, when it is the only function imported:
 * - Lodash: 11,113 bytes
 * - Micro-dash: 662 bytes
 */
export declare function isMatch<T>(object: T, source: DeepPartial<T>): boolean;
