import { NonUndefined } from 'utility-types';
import { Existent, Key, Nil } from '../interfaces';
/** @hidden */
declare type Fn = (...args: any[]) => any;
/** @hidden */
declare type Obj1<K1 extends Key, V> = {
    [key in K1]?: V;
};
/** @hidden */
declare type Path1<K1 extends Key, T extends Obj1<K1, any> | Nil> = T extends Existent ? T[K1] : NonNullable<T>[K1] | undefined;
/** @hidden */
declare type DefinedPath1<K1 extends Key, T extends Obj1<K1, any> | Nil> = NonUndefined<NonNullable<T>[K1]>;
/** @hidden */
declare type Obj2<K1 extends Key, K2 extends Key, V> = {
    [k1 in K1]?: Obj1<K2, V>;
};
/** @hidden */
declare type Path2<K1 extends Key, K2 extends Key, T extends Obj2<K1, K2, any> | Nil> = Path1<K2, Path1<K1, T>>;
/** @hidden */
declare type DefinedPath2<K1 extends Key, K2 extends Key, T extends Obj2<K1, K2, any> | Nil> = DefinedPath1<K2, DefinedPath1<K1, T>>;
/** @hidden */
declare type Obj3<K1 extends Key, K2 extends Key, K3 extends Key, V> = {
    [k1 in K1]?: Obj2<K2, K3, V>;
};
/** @hidden */
declare type Path3<K1 extends Key, K2 extends Key, K3 extends Key, T extends Obj3<K1, K2, K3, any> | Nil> = Path1<K3, Path2<K1, K2, T>>;
/** @hidden */
declare type DefinedPath3<K1 extends Key, K2 extends Key, K3 extends Key, T extends Obj3<K1, K2, K3, any> | Nil> = DefinedPath1<K3, DefinedPath2<K1, K2, T>>;
/**
 * Invokes the method at `path` of `object`.
 *
 * Differences from lodash:
 * - only accepts an array for `path`, not a dot-separated string
 *
 * Contribution to minified bundle size, when it is the only function imported:
 * - Lodash: 6,245 bytes
 * - Micro-dash: 273 bytes
 */
export declare function invoke<K1 extends Key, K2 extends Key, K3 extends Key, T extends Obj3<K1, K2, K3, Fn> | Nil>(object: T, path: [K1, K2, K3], ...args: Parameters<DefinedPath3<K1, K2, K3, T>>): Path3<K1, K2, K3, T> extends DefinedPath3<K1, K2, K3, T> ? ReturnType<DefinedPath3<K1, K2, K3, T>> : ReturnType<DefinedPath3<K1, K2, K3, T>> | undefined;
export declare function invoke<K1 extends Key, K2 extends Key, T extends Obj2<K1, K2, Fn> | Nil>(object: T, path: [K1, K2], ...args: Parameters<DefinedPath2<K1, K2, T>>): Path2<K1, K2, T> extends DefinedPath2<K1, K2, T> ? ReturnType<DefinedPath2<K1, K2, T>> : ReturnType<DefinedPath2<K1, K2, T>> | undefined;
export declare function invoke<K1 extends Key, T extends Obj1<K1, Fn> | Nil>(object: T, path: [K1], ...args: Parameters<DefinedPath1<K1, T>>): Path1<K1, T> extends DefinedPath1<K1, T> ? ReturnType<DefinedPath1<K1, T>> : ReturnType<DefinedPath1<K1, T>> | undefined;
export declare function invoke(object: object | Nil, path: []): undefined;
export declare function invoke(object: object | Nil, path: Key[], ...args: any[]): any;
export {};
